
#include "stdafx.h"
#include "HashCalc.h"

int AnsiToUnicode(/*in*/ const char* szSrc, /*out*/ WCHAR* wcDst, /*in*/ DWORD dwDstLen)
{
	return MultiByteToWideChar(CP_ACP, 0, szSrc, strlen(szSrc), wcDst, dwDstLen);
}

int UnicodeToAnsi(/*in*/ WCHAR* wcSrc, /*out*/ char* szDst, /*in*/ DWORD dwDstLen)
{
	return WideCharToMultiByte(CP_ACP, 0, wcSrc, wcslen(wcSrc), szDst, dwDstLen, NULL, NULL);
}

HashCalc::HashCalc(ALGORITHM_TYPE AlgorithmType)
{
	m_Type = AlgorithmType;
	m_dwErr = 0;
	m_bExit = FALSE;
}

HashCalc::~HashCalc()
{

}

tstring HashCalc::string_crc32(string strInputA, string strKeyA, BOOL bUppercase)
{
	CRC32 crc;
	crc.Restart();
	crc.Update((const byte*)strInputA.c_str(), strInputA.length());

	byte output[4] = { 0 };
	crc.TruncatedFinal(output, 4);

	tstring strResult;
	for (int i = 3; i >= 0; i--)
	{
		TCHAR szBuf[5] = { 0 };
		_stprintf(szBuf, bUppercase ? 
			_T("%02X") : _T("%02x"), output[i]);
		strResult += szBuf;
	}

	return strResult;
}

tstring HashCalc::string_adler32(string strInputA, string strKeyA, BOOL bUppercase)
{
	Adler32 h;
	h.Restart();
	h.Update((const byte*)strInputA.c_str(), strInputA.length());

	byte output[4] = { 0 };
	h.TruncatedFinal(output, 4);

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 4);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_tiger(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[24] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Tiger> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 24);
	}
	else
	{
		Tiger h;
		h.Restart();
		h.Update((const byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 24);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 24);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_panama(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[32] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC< Weak::PanamaHash<LittleEndian> > h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 32);
	}
	else
	{
		Weak::PanamaHash<LittleEndian> h;
		h.Restart();
		h.Update((const byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 32);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 32);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_whirlpool(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[64] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Whirlpool> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 64);
	}
	else
	{
		Whirlpool h;
		h.Restart();
		h.Update((const byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 64);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 64);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_md2(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[16] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Weak1::MD2> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}
	else
	{

		Weak1::MD2 h;
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 16);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_md4(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[16] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Weak1::MD4> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}
	else
	{

		Weak1::MD4 h;
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 16);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_md5(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[16] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Weak1::MD5> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}
	else
	{

		Weak1::MD5 h;
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 16);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_ripemd_128(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[16] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD128> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}
	else
	{
		RIPEMD128 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 16);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 16);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_ripemd_160(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[20] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD160> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 20);
	}
	else
	{
		RIPEMD160 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 20);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 20);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_ripemd_256(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[32] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD256> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 32);
	}
	else
	{
		RIPEMD256 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 32);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 32);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_ripemd_320(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[40] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD320> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 40);
	}
	else
	{
		RIPEMD320 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 40);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 40);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_sha1(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[20] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA1> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 20);
	}
	else
	{
		SHA1 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 20);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 20);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_sha2_224(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[28] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA224> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 28);
	}
	else
	{
		SHA224 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 28);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 28);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_sha2_256(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[32] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA256> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 32);
	}
	else
	{
		SHA256 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 32);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 32);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_sha4_384(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[48] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA384> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 48);
	}
	else
	{
		SHA384 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 48);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 48);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::string_sha4_512(string strInputA, string strKeyA, BOOL bHmac, BOOL bUppercase)
{
	byte output[64] = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA512> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 64);
	}
	else
	{
		SHA512 h;
		h.Restart();
		h.Update((byte*)strInputA.c_str(), strInputA.length());
		h.TruncatedFinal(output, 64);
	}

	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 64);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
}

tstring HashCalc::file_crc32(TCHAR* szInFile, string strKeyA, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);

	CRC32 crc;
	crc.Restart();
	byte output[4]     = { 0 };

	while (dwReadLen)
	{
		if (m_bExit)
			goto _EXIT;

		crc.Update((const byte*)ReadBuf, dwReadLen);
		CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
		dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
	}	crc.TruncatedFinal(output, 4);

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		tstring strResult;
		for (int i = 3; i >= 0; i--)
		{
			TCHAR szBuf[5] = { 0 };
			_stprintf(szBuf, bUppercase ? 
				_T("%02X") : _T("%02x"), output[i]);
			strResult += szBuf;
		}
		return strResult;
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_Adler32(TCHAR* szInFile, string strKeyA, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);

	Adler32 h;
	h.Restart();
	byte output[4]     = { 0 };

	while (dwReadLen)
	{		
		if (m_bExit)
			goto _EXIT;
		h.Update((const byte*)ReadBuf, dwReadLen);
		CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
		dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
	}	h.TruncatedFinal(output, 4);

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 4);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_tiger(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[24]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Tiger> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 24);
	}
	else
	{
		Tiger h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 24);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 24);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_panama(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[32]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC< Weak::PanamaHash<LittleEndian> > h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 32);
	}
	else
	{
		Weak::PanamaHash<LittleEndian> h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 32);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 32);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_whirlpool(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[64]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Whirlpool> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 64);
	}
	else
	{
		Whirlpool h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 64);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 64);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_md2(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[16]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Weak1::MD2> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}
	else
	{
		Weak1::MD2 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 16);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_md4(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[16]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Weak1::MD4> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}
	else
	{
		Weak1::MD4 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 16);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_md5(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[16]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<Weak1::MD5> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}
	else
	{
		Weak1::MD5 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 16);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_ripemd_128(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[16]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD128> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}
	else
	{
		RIPEMD128 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 16);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 16);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_ripemd_160(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[20]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD160> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 20);
	}
	else
	{
		RIPEMD160 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 20);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 20);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_ripemd_256(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[32]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD256> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 32);
	}
	else
	{
		RIPEMD256 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 32);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 32);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_ripemd_320(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[40]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<RIPEMD320> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 40);
	}
	else
	{
		RIPEMD320 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 40);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 40);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_sha1(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[20]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA1> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 20);
	}
	else
	{
		SHA1 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 20);
	}

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 20);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_sha2_224(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[28]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA224> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 28);
	}
	else
	{
		SHA224 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 28);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 28);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_sha2_256(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[32]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA256> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 32);
	}
	else
	{
		SHA256 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 32);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 32);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_sha4_384(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[48]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA384> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 48);
	}
	else
	{
		SHA384 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 48);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
		string strResult;
		HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
		encoder.Put(output, 48);
		encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

tstring HashCalc::file_sha4_512(TCHAR* szInFile, string strKeyA, BOOL bHmac, BOOL bUppercase, MemberFxn addr)
{
	File Fin;
	if (!Fin.Open((LPTSTR)(LPCTSTR)szInFile))
	{
		m_dwErr = Fin.GetErrorCode();
		return _T("");
	}

	DWORD64 dwFileSize = Fin.GetFileSize();
	char* ReadBuf      = new char[_BUFFER_SIZE];
	DWORD dwReadLen    = Fin.Read(ReadBuf, _BUFFER_SIZE);
	byte output[64]    = { 0 };

	if (bHmac && !strKeyA.empty())
	{
		HMAC<SHA512> h;
		h.Restart();
		h.SetKey((byte*)strKeyA.c_str(), strKeyA.length());

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;

			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 64);
	}
	else
	{
		SHA512 h;
		h.Restart();

		while (dwReadLen)
		{
			if (m_bExit)
				goto _EXIT;
			h.Update((const byte*)ReadBuf, dwReadLen);
			CallMember(addr, CalcProgress(dwFileSize, Fin.GetPosition()));
			dwReadLen = Fin.Read(ReadBuf, _BUFFER_SIZE);
		}	h.TruncatedFinal(output, 64);
	}

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;

	{
	string strResult;
	HexEncoder encoder(new StringSink(strResult), bUppercase ? true : false);
	encoder.Put(output, 64);
	encoder.MessageEnd();

#ifdef _UNICODE
	WCHAR szBuffer[200] = { 0 };
	AnsiToUnicode(strResult.c_str(), szBuffer, 200);
	return tstring(szBuffer);
#else
	return strResult;
#endif
	}

_EXIT:

	delete ReadBuf;
	Fin.Close();
	m_dwErr = 0;
	return _T("");
}

DWORD HashCalc::CalcProgress(DWORD64 dwFileSize, DWORD64 dwCurrent)
{
	if (dwFileSize / 100 > 0)
		return (dwCurrent / (dwFileSize / 100));
	return 100;
}

void HashCalc::CallMember(MemberFxn addr, int nProgress/* 1-100 */)
{
	if (addr.IsNull())
		return;
	
	DWORD dwAdddr = addr.GetAddr();
	PVOID pThis = addr.GetThis();

	__asm mov ecx, pThis;
	__asm push nProgress;
	__asm call dwAdddr;
}

DWORD HashCalc::GetError()
{
	return m_dwErr;
}

tstring HashCalc::GetStringResult(string strInputA, // ַANSI
	string strKeyA,                       // ַANSI
	BOOL bHmac, BOOL bUppercase)
{
	switch (m_Type)
	{
	case E_CRC32:
		return string_crc32(strInputA, strKeyA, bUppercase);
	case E_ADLER32:
		return string_adler32(strInputA, strKeyA, bUppercase);
	case E_TIGER:
		return string_tiger(strInputA, strKeyA, bHmac, bUppercase);
	case E_PANAMA:
		return string_panama(strInputA, strKeyA, bHmac, bUppercase);
	case E_WHIRLPOOL:
		return string_whirlpool(strInputA, strKeyA, bHmac, bUppercase);
	case E_MD2:
		return string_md2(strInputA, strKeyA, bHmac, bUppercase);
	case E_MD4:
		return string_md4(strInputA, strKeyA, bHmac, bUppercase);
	case E_MD5:
		return string_md5(strInputA, strKeyA, bHmac, bUppercase);
	case E_RIPEMD_128:
		return string_ripemd_128(strInputA, strKeyA, bHmac, bUppercase);
	case E_RIPEMD_160:
		return string_ripemd_160(strInputA, strKeyA, bHmac, bUppercase);
	case E_RIPEMD_256:
		return string_ripemd_256(strInputA, strKeyA, bHmac, bUppercase);
	case E_RIPEMD_320:
		return string_ripemd_320(strInputA, strKeyA, bHmac, bUppercase);
	case E_SHA1:
		return string_sha1(strInputA, strKeyA, bHmac, bUppercase);
	case E_SHA2_224:
		return string_sha2_224(strInputA, strKeyA, bHmac, bUppercase);
	case E_SHA2_256:
		return string_sha2_256(strInputA, strKeyA, bHmac, bUppercase);
	case E_SHA4_384:
		return string_sha4_384(strInputA, strKeyA, bHmac, bUppercase);
	case E_SHA4_512:
		return string_sha4_512(strInputA, strKeyA, bHmac, bUppercase);
	default:
		return tstring(_T(""));
	}
}

tstring HashCalc::GetFileResult(TCHAR* szInFile, 
	string strKeyA, BOOL bHmac, 
	BOOL bUppercase, MemberFxn addr)
{
	m_bExit = FALSE;

	switch (m_Type)
	{
	case E_CRC32:
		return file_crc32(szInFile, strKeyA, bUppercase, addr);
	case E_ADLER32:
		return file_Adler32(szInFile, strKeyA, bUppercase, addr);
	case E_TIGER:
		return file_tiger(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_PANAMA:
		return file_panama(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_WHIRLPOOL:
		return file_whirlpool(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_MD2:
		return file_md2(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_MD4:
		return file_md4(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_MD5:
		return file_md5(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_RIPEMD_128:
		return file_ripemd_128(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_RIPEMD_160:
		return file_ripemd_160(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_RIPEMD_256:
		return file_ripemd_256(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_RIPEMD_320:
		return file_ripemd_320(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_SHA1:
		return file_sha1(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_SHA2_224:
		return file_sha2_224(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_SHA2_256:
		return file_sha2_256(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_SHA4_384:
		return file_sha4_384(szInFile, strKeyA, bHmac, bUppercase, addr);
	case E_SHA4_512:
		return file_sha4_512(szInFile, strKeyA, bHmac, bUppercase, addr);
	default:
		return tstring(_T(""));
	}
}