
/*++

Autor: Zhang Luduo (³)
Email: Support@ZhangLuduo.com 
MSN  : ZhangLuduo@msn.com
WEB  : http://www.ZhangLuduo.com/

Copyright 2006-2009 Zhang Luduo. All rights reserved.

--*/

#ifndef _HYPERLINK_H
#define _HYPERLINK_H

#ifndef _INC_WINDOWS
#	include <windows.h>
#endif

#ifndef _INC_TCHAR
#	include <tchar.h>
#endif

#include "Thunk.h"
//#pragma comment(lib, "Thunk.lib")

#include <vector>
#include "tstring.h"

class HyperLink
{
public:

	typedef enum _STATUS_COLOR
	{
		NORMAL,
		HOVER,
		ACTIVE,
		VISITED		
	}	STATUS_COLOR;

	typedef enum _UNDERLINE_SHOW
	{
		ALWAYS,
		MOUSEMOVE,
		NOSHOW
	}	UNDERLINE_SHOW;

	typedef struct _COLOR_INFO
	{
		COLORREF Normal;
		COLORREF Hover;
		COLORREF Active;
		COLORREF Visited;
	}	COLOR_INFO;

	typedef struct _FONT_INTO
	{
		LOGFONT LogFont;
		HFONT StdFont;
		HFONT UnderlineFont;
		HFONT OriginalFont;
	}	FONT_INTO;

	typedef struct _SUBWND_INTO
	{
		HWND hWnd;
		DWORD dwRef;
	}	SUBWND_INTO;

public:

	//
	// Construction and destructin
	//

	HyperLink();
	virtual ~HyperLink();

	//
	// Install and uninstall
	//

	BOOL Attach(HWND hWnd);
	BOOL Detach();
	operator HWND() const;
	
	//
	// Set attribute
	//

	HyperLink& SetText(TCHAR* strText);
	HyperLink& SetFontBold(BOOL bBold);
	HyperLink& SetFontItalic(BOOL bItalic);
	HyperLink& SetFontSize(int nSize);
	HyperLink& SetFontName(TCHAR* strFontName);
	HyperLink& SetURL(TCHAR* strURL);
	HyperLink& SetFont(HFONT hFont);
	HyperLink& SetCursor(HCURSOR hCursor);
	HyperLink& SetColor(COLORREF Color, STATUS_COLOR StatusColor);
	HyperLink& SetUnderlineShow(UNDERLINE_SHOW UnderlineShow);

protected:

	// Overridables
	virtual void OnNavigate(); 

protected:

	LRESULT HyperLinkProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	LRESULT ParentWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	HyperLink& AdjustFit();
	HyperLink& CreateFont();
	HyperLink& UpdateFont();
	HyperLink& DeleteFont();

protected:

	HWND m_hWnd;

	Thunk   m_pHyperLinkProc;
	Thunk   m_pParentWndProc;
	WNDPROC m_PrevHyperLinkProc;
	WNDPROC m_PrevParentWndProc;

	BOOL m_bHover; 
	BOOL m_bVisited; 
	BOOL m_LButtonDown;

	UNDERLINE_SHOW m_UnderlineShow;
	COLOR_INFO m_ColorInfo;
	FONT_INTO  m_FontInfo;

	tstring m_strText;
	tstring m_strURL;
	HCURSOR m_hCursor;

	static vector<SUBWND_INTO> m_SubwndInfo;
};

#endif // _HYPERLINK_H
